@@ -262,6 +262,11 @@ ags_ffplayer_init(AgsFFPlayer *ffplayer)
   ffplayer->mapped_input_pad = 0;
   ffplayer->mapped_output_pad = 0;
 
+#if 0
+  ags_machine_popup_add_edit_options((AgsMachine *) ffplayer,
+				     (AGS_MACHINE_POPUP_ENVELOPE));
+#endif
+				     
   ffplayer->name = NULL;
   ffplayer->xml_type = "ags-ffplayer\0";
 
