@@ -271,8 +271,14 @@ ags_matrix_init(AgsMatrix *matrix)
   matrix->mapped_input_pad = 0;
   matrix->mapped_output_pad = 0;
 
+#if 0
   ags_machine_popup_add_edit_options((AgsMachine *) matrix,
-				     (AGS_MACHINE_POPUP_COPY_PATTERN));
+				     (AGS_MACHINE_POPUP_COPY_PATTERN |
+				      AGS_MACHINE_POPUP_ENVELOPE));
+#else
+  ags_machine_popup_add_edit_options((AgsMachine *) matrix,
+    (AGS_MACHINE_POPUP_COPY_PATTERN));
+#endif
   
   /* create widgets */
   frame = (GtkFrame *) (gtk_bin_get_child((GtkBin *) matrix));
