@@ -25,6 +25,8 @@
 
 #include <fcntl.h>
 #include <sys/stat.h>
+#include <stdlib.h>
+#include <string.h>
 
 void ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder);
 void ags_midi_builder_init(AgsMidiBuilder *midi_builder);
@@ -264,9 +266,6 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::midi-putc signal is emited during putting char to file.
    *
-<<<<<<< HEAD
-   * Since: 1.0.0
-=======
    * Since: 0.7.90
    */
   midi_builder_signals[MIDI_PUTC] =
@@ -475,6 +474,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
   /**
    * AgsMidiBuilder::change-program:
    * @midi_builder: the builder
+   * @delta_time: delta-time
    * @channel: the audio channel
    * @program: the new programm
    *
@@ -488,14 +488,16 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_change_program),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT,
-		 G_TYPE_NONE, 2,
+		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT,
+		 G_TYPE_NONE, 3,
+		 G_TYPE_UINT,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT);
 
   /**
    * AgsMidiBuilder::change-channel-pressure:
    * @midi_builder: the builder
+   * @delta_time: delta-time
    * @channel: the audio channel
    * @pressure: the new pressure, aftertouch
    *
@@ -509,14 +511,16 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_change_pressure),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT,
-		 G_TYPE_NONE, 2,
+		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT,
+		 G_TYPE_NONE, 3,
+		 G_TYPE_UINT,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT);
 
   /**
    * AgsMidiBuilder::sysex:
    * @midi_builder: the builder
+   * @delta_time: delta-time
    * @sysex_data: the data array
    * @length: the length of the array
    *
@@ -530,8 +534,9 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_sysex),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__POINTER_UINT,
-		 G_TYPE_NONE, 2,
+		 g_cclosure_user_marshal_VOID__UINT_POINTER_UINT,
+		 G_TYPE_NONE, 3,
+		 G_TYPE_UINT,
 		 G_TYPE_POINTER,
 		 G_TYPE_UINT);
 
@@ -558,9 +563,10 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_TYPE_UINT,
 		 G_TYPE_UINT);
 
-    /**
+  /**
    * AgsMidiBuilder::song-position:
    * @midi_builder: the builder
+   * @delta_time: delta-time
    * @song_position: the current position from 0x0 to 0x3fff
    *
    * The ::song-position signal is emited during building of event.
@@ -573,13 +579,15 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_song_position),
 		 NULL, NULL,
-		 g_cclosure_marshal_VOID__UINT,
-		 G_TYPE_NONE, 1,
+		 g_cclosure_user_marshal_VOID__UINT_UINT,
+		 G_TYPE_NONE, 2,
+		 G_TYPE_UINT,
 		 G_TYPE_UINT);
 
-    /**
+  /**
    * AgsMidiBuilder::song-select:
    * @midi_builder: the builder
+   * @delta_time: delta-time
    * @song_select: the song or sequence
    *
    * The ::song-select signal is emited during building of event.
@@ -592,13 +600,15 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_song_select),
 		 NULL, NULL,
-		 g_cclosure_marshal_VOID__UINT,
-		 G_TYPE_NONE, 1,
+		 g_cclosure_user_marshal_VOID__UINT_UINT,
+		 G_TYPE_NONE, 2,
+		 G_TYPE_UINT,
 		 G_TYPE_UINT);
 
-    /**
+  /**
    * AgsMidiBuilder::tune-request:
    * @midi_builder: the builder
+   * @delta_time: delta-time
    *
    * The ::tune-request signal is emited during building of event.
    *
@@ -610,13 +620,15 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_tune_request),
 		 NULL, NULL,
-		 g_cclosure_marshal_VOID__VOID,
-		 G_TYPE_NONE, 0);
+		 g_cclosure_marshal_VOID__UINT,
+		 G_TYPE_NONE, 1,
+		 G_TYPE_UINT);
 
 
   /**
    * AgsMidiBuilder::sequence-number:
    * @midi_builder: the builder
+   * @delta_time: delta-time
    * @sequence: the nth sequence
    *
    * The ::sequence-number signal is emited during building of event.
@@ -629,13 +641,15 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_sequence_number),
 		 NULL, NULL,
-		 g_cclosure_marshal_VOID__UINT,
-		 G_TYPE_NONE, 1,
+		 g_cclosure_user_marshal_VOID__UINT_UINT,
+		 G_TYPE_NONE, 2,
+		 G_TYPE_UINT,
 		 G_TYPE_UINT);
   
   /**
    * AgsMidiBuilder::smtpe:
    * @midi_builder: the builder
+   * @delta_time: delta-time
    * @hr: hours
    * @mn: minutes
    * @se: seconds
@@ -664,6 +678,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
   /**
    * AgsMidiBuilder::tempo:
    * @midi_builder: the builder
+   * @delta_time: delta-time
    * @tempo: the tempo number as 24-bit quantifier
    *
    * The ::tempo signal is emited during building of event.
@@ -676,8 +691,9 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_tempo),
 		 NULL, NULL,
-		 g_cclosure_marshal_VOID__UINT,
-		 G_TYPE_NONE, 1,
+		 g_cclosure_user_marshal_VOID__UINT_UINT,
+		 G_TYPE_NONE, 2,
+		 G_TYPE_UINT,
 		 G_TYPE_UINT);
 
   /**
@@ -699,7 +715,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_time_signature),
 		 NULL, NULL,
-		 g_cclosure_marshal_VOID__UINT,
+		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 5,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -733,6 +749,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
   /**
    * AgsMidiBuilder::sequencer-meta-event:
    * @midi_builder: the builder
+   * @delta_time: delta-time
    * @len: length
    * @id: identifier
    * @data: buffer
@@ -747,8 +764,9 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_sequencer_meta_event),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT,
-		 G_TYPE_NONE, 3,
+		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT,
+		 G_TYPE_NONE, 4,
+		 G_TYPE_UINT,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT);
@@ -756,6 +774,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
   /**
    * AgsMidiBuilder::text-event:
    * @midi_builder: the builder
+   * @delta_time: delta-time
    * @text: the text
    * @length: length
    *
@@ -769,9 +788,10 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_text_event),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT,
-		 G_TYPE_NONE, 2,
+		 g_cclosure_user_marshal_VOID__UINT_STRING_UINT,
+		 G_TYPE_NONE, 3,
 		 G_TYPE_UINT,
+		 G_TYPE_STRING,
 		 G_TYPE_UINT);
 }
 
@@ -782,7 +802,7 @@ ags_midi_builder_init(AgsMidiBuilder *midi_builder)
 
   midi_builder->data = NULL;
   midi_builder->length = 0;
-  
+
   midi_builder->file = NULL;
   
   midi_builder->midi_header = NULL;
@@ -864,6 +884,9 @@ ags_midi_builder_header_alloc()
   midi_builder_header->times = 0;
   midi_builder_header->beat = 0;
   midi_builder_header->clicks = 0;
+
+  midi_builder_header->data = NULL;
+  midi_builder_header->length = 0;
   
   return(midi_builder_header);
 }
@@ -905,7 +928,7 @@ ags_midi_builder_track_alloc()
   midi_builder_track->offset = 0;
   midi_builder_track->track_name = NULL;
   
-  midi_builder_track->delta_time = 0;
+  midi_builder_track->absolute_time = 0;
   
   midi_builder_track->data = NULL;
   midi_builder_track->length = 0;
@@ -942,7 +965,7 @@ ags_midi_builder_track_free(AgsMidiBuilderTrack *midi_builder_track)
 /**
  * ags_midi_builder_track_find_track_name_with_delta_time:
  * @midi_builder_track: the #AgsMidiBuilderTrack
- * @delta_time: the delta time
+ * @absolute_time: the absolute time
  * @track_name: the track's string representation
  * 
  * Finds matching #AgsMidiBuilderTrack-struct within @midi_builder_track #GList-struct.
@@ -953,11 +976,11 @@ ags_midi_builder_track_free(AgsMidiBuilderTrack *midi_builder_track)
  */
 GList*
 ags_midi_builder_track_find_delta_time_with_track_name(GList *midi_builder_track,
-						       guint delta_time,
+						       guint64 absolute_time,
 						       gchar *track_name)
 {
   while(midi_builder_track != NULL){
-    if(AGS_MIDI_BUILDER_TRACK(midi_builder_track->data)->delta_time == delta_time &&
+    if(AGS_MIDI_BUILDER_TRACK(midi_builder_track->data)->absolute_time == absolute_time &&
        !g_ascii_strcasecmp(AGS_MIDI_BUILDER_TRACK(midi_builder_track->data)->track_name,
 			   track_name)){
       break;
@@ -982,9 +1005,10 @@ void
 ags_midi_builder_track_insert_midi_message(AgsMidiBuilderTrack *midi_builder_track,
 					   unsigned char *buffer, guint length)
 {
-  unsigned char *current, *new_buffer;
+  unsigned char *current;
   
-  guint prefix_length, suffix_length;
+  gint prefix_length, suffix_length;
+  guint64 absolute_time;
   glong delta_time;
   
   if(midi_builder_track == NULL ||
@@ -997,48 +1021,45 @@ ags_midi_builder_track_insert_midi_message(AgsMidiBuilderTrack *midi_builder_tra
   ags_midi_buffer_util_get_varlength(buffer,
 				     &delta_time);
 
+  midi_builder_track->absolute_time += delta_time;
+  
   /* get offset */
-  //TODO:JK: probably you should use realloc
-  current = ags_midi_builder_track_get_delta_time_offset(midi_builder_track,
-							 delta_time);
-  new_buffer = (unsigned char *) malloc((midi_builder_track->length + length) * sizeof(unsigned char));
+  midi_builder_track->data = (unsigned char *) realloc(midi_builder_track->data,
+						       (midi_builder_track->length + length) * sizeof(unsigned char));
+
+  current = midi_builder_track->data + midi_builder_track->length;
   
   if(current == NULL){
     prefix_length = 0;
     suffix_length = 0;
   }else{
-    /* prefix */
-    prefix_length = current - buffer;
-
-    if(prefix_length > 0){
-      memcpy(new_buffer, buffer, prefix_length);
+    /* prefix */    
+    if(current <= midi_builder_track->data + midi_builder_track->length){
+      prefix_length = current - midi_builder_track->data;
+    }else{
+      prefix_length = 0;
     }
 
     /* suffix */
-    suffix_length = midi_builder_track->length - prefix_length;
+    if(midi_builder_track->length > prefix_length){
+      suffix_length = midi_builder_track->length - prefix_length;
 
-    if(suffix_length > 0){
-      memcpy(new_buffer + prefix_length + length, buffer, suffix_length);
+      memmove(midi_builder_track->data + prefix_length + length, midi_builder_track->data + prefix_length, suffix_length);
     }
-
-    current = new_buffer;
   }
 
-  memcpy(new_buffer + prefix_length, buffer, length);
+  memcpy(midi_builder_track->data + prefix_length, buffer, length);
 
   /* set data */
-  if(midi_builder_track->data != NULL){
-    free(midi_builder_track->data);
-  }
-  
-  midi_builder_track->data = new_buffer;
   midi_builder_track->length += length;
+  ags_midi_buffer_util_put_int32(midi_builder_track->data + 4,
+				 midi_builder_track->length);
 }
 
 /**
  * ags_midi_builder_track_get_delta_time_offset:
  * @midi_builder_track: the #AgsMidiBuilderTrack-struct
- * @delta_time: the delta-time
+ * @absolute_time: the absolute time
  * 
  * Get offset by delta time.
  * 
@@ -1048,39 +1069,43 @@ ags_midi_builder_track_insert_midi_message(AgsMidiBuilderTrack *midi_builder_tra
  */
 unsigned char*
 ags_midi_builder_track_get_delta_time_offset(AgsMidiBuilderTrack *midi_builder_track,
-					     guint delta_time)
+					     guint64 absolute_time)
 {
-  unsigned char *buffer, *prev, *current;
+  unsigned char *prev, *current;
 
-  guint current_delta_time;
+  guint64 time_counter;
+  glong current_delta_time;
   
   if(midi_builder_track == NULL ||
      midi_builder_track->data == NULL){
     return(NULL);
   }
 
-  buffer =
-    current = midi_builder_track->data;
-
-  prev = NULL;
+  current = midi_builder_track->data;
   
-  while(current < buffer + midi_builder_track->length){
+  prev = NULL;
+  time_counter = 0;
+  
+  while(current != NULL){
     current = ags_midi_buffer_util_seek_message(current,
 						1,
 						&current_delta_time);
-
-    if(current_delta_time > delta_time){
-      if(prev != NULL){
-	current = prev;
-      }else{
-	current = buffer;
-      }
-      
+    time_counter += current_delta_time;
+    
+    if(time_counter > absolute_time){
+      break;
+    }
+   
+    if(current >= midi_builder_track->data + midi_builder_track->length){
       break;
     }
 
     prev = current;
   }
+
+  if(current == NULL){
+    current = midi_builder_track->data + midi_builder_track->length;
+  }
   
   return(current);
 }
@@ -1230,8 +1255,8 @@ ags_midi_builder_real_append_track(AgsMidiBuilder *midi_builder,
 
   /* allocate struct */
   midi_builder_track = ags_midi_builder_track_alloc();
-  midi_builder_track->track_name = track_name;
-
+  midi_builder_track->track_name = g_strdup(track_name);
+  
   /* fill buffer */
   delta_time_size = ags_midi_buffer_util_get_varlength_size(0);
 
@@ -1242,7 +1267,7 @@ ags_midi_builder_real_append_track(AgsMidiBuilder *midi_builder,
   }
   
   midi_builder_track->data = (unsigned char *) malloc((8 + delta_time_size + length + 3) * sizeof(unsigned char));
-  midi_builder_track->length = 8 + delta_time_size + length + 3;
+  midi_builder_track->length = (4 + 4) + delta_time_size + length + 3;
 
   ags_midi_buffer_util_put_track(midi_builder_track->data,
 				 0);
@@ -1293,7 +1318,7 @@ ags_midi_builder_real_append_key_on(AgsMidiBuilder *midi_builder,
   guint delta_time_size;
 
   delta_time_size = ags_midi_buffer_util_get_varlength_size(delta_time);
-  
+
   buffer = (unsigned char *) malloc((delta_time_size + 3) * sizeof(unsigned char));
   ags_midi_buffer_util_put_key_on(buffer,
 				  delta_time,
@@ -1974,12 +1999,12 @@ ags_midi_builder_real_append_tempo(AgsMidiBuilder *midi_builder,
 
   delta_time_size = ags_midi_buffer_util_get_varlength_size(delta_time);
   
-  buffer = (unsigned char *) malloc((delta_time_size + 3) * sizeof(unsigned char));
+  buffer = (unsigned char *) malloc((delta_time_size + 6) * sizeof(unsigned char));
   ags_midi_buffer_util_put_tempo(buffer,
 				 delta_time,
 				 tempo);
   ags_midi_builder_track_insert_midi_message(midi_builder->current_midi_track,
-					     buffer, delta_time_size + 3);
+					     buffer, delta_time_size + 6);
 }
 
 
@@ -2155,6 +2180,12 @@ ags_midi_builder_real_append_text_event(AgsMidiBuilder *midi_builder,
 
   guint delta_time_size;
 
+  if(length > 0){
+    if(text[length - 1] == '\0'){
+      length--;
+    }
+  }
+  
   delta_time_size = ags_midi_buffer_util_get_varlength_size(delta_time);
   
   buffer = (unsigned char *) malloc((delta_time_size + length + 3) * sizeof(unsigned char));
@@ -3063,6 +3094,7 @@ ags_midi_builder_build(AgsMidiBuilder *midi_builder)
 
   while(midi_track != NULL){
     length += AGS_MIDI_BUILDER_TRACK(midi_track->data)->length;
+    length += 4; //NOTE:JK: EOT
     
     midi_track = midi_track->next;
   }
@@ -3074,8 +3106,14 @@ ags_midi_builder_build(AgsMidiBuilder *midi_builder)
   
   if(length > 0){
     midi_builder->data = (unsigned char *) malloc(length * sizeof(unsigned char));
+
+    midi_builder->length = length;
   }else{
     midi_builder->data = NULL;
+    
+    midi_builder->length = 0;
+
+    return;
   }
 
   /* fill */
@@ -3084,6 +3122,7 @@ ags_midi_builder_build(AgsMidiBuilder *midi_builder)
   if(midi_builder->midi_header != NULL &
      midi_builder->midi_header->data != NULL){
     memcpy(offset, midi_builder->midi_header->data, midi_builder->midi_header->length * sizeof(unsigned char));
+    offset += midi_builder->midi_header->length;
   }
   
   midi_track = midi_builder->midi_track;
@@ -3091,6 +3130,11 @@ ags_midi_builder_build(AgsMidiBuilder *midi_builder)
   while(midi_track != NULL){
     if(AGS_MIDI_BUILDER_TRACK(midi_track->data)->data != NULL){
       memcpy(offset, AGS_MIDI_BUILDER_TRACK(midi_track->data)->data, AGS_MIDI_BUILDER_TRACK(midi_track->data)->length * sizeof(unsigned char));
+      offset += AGS_MIDI_BUILDER_TRACK(midi_track->data)->length;
+      
+      ags_midi_buffer_util_put_end_of_track(offset,
+					    0);
+      offset += 4;
     }
   
     midi_track = midi_track->next;
