@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2015 Joël Krähemann
+ * Copyright (C) 2005-2017 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -20,6 +20,8 @@
 #include <ags/audio/recall/ags_envelope_audio_signal.h>
 #include <ags/audio/recall/ags_envelope_channel.h>
 
+#include <ags/lib/ags_complex.h>
+
 #include <ags/object/ags_connectable.h>
 #include <ags/object/ags_dynamic_connectable.h>
 #include <ags/object/ags_soundcard.h>
@@ -34,6 +36,7 @@
 #include <ags/audio/ags_recall_channel_run.h>
 
 #include <stdlib.h>
+#include <complex.h>
 
 void ags_envelope_audio_signal_class_init(AgsEnvelopeAudioSignalClass *envelope_audio_signal);
 void ags_envelope_audio_signal_connectable_interface_init(AgsConnectableInterface *connectable);
@@ -214,44 +217,6 @@ ags_envelope_audio_signal_finalize(GObject *gobject)
   G_OBJECT_CLASS(ags_envelope_audio_signal_parent_class)->finalize(gobject);
 }
 
-void
-ags_envelope_audio_signal_run_inter(AgsRecall *recall)
-{
-  AgsEnvelopeChannel *envelope_channel;
-  AgsEnvelopeAudioSignal *envelope_audio_signal;
-  AgsAudioSignal *source;
-  GList *stream_source;
-  gboolean audio_enveloped, channel_enveloped;
-  guint buffer_size;
-  guint i;
-  GValue audio_value = {0,};
-  GValue channel_value = {0,};
-
-  AGS_RECALL_CLASS(ags_envelope_audio_signal_parent_class)->run_inter(recall);
-
-  envelope_audio_signal = AGS_ENVELOPE_AUDIO_SIGNAL(recall);
-
-  source = AGS_RECALL_AUDIO_SIGNAL(envelope_audio_signal)->source;
-  stream_source = source->stream_current;
-  buffer_size = source->buffer_size;
-
-  if(stream_source == NULL){
-    ags_recall_done(recall);
-    return;
-  }
-
-  /* check channel */
-  envelope_channel = AGS_ENVELOPE_CHANNEL(AGS_RECALL_CHANNEL_RUN(recall->parent->parent)->recall_channel);
-
-  //FIXME:JK: access ports safe
-  ags_audio_signal_envelope(source,
-			    envelope_channel->attack->port_value.ags_port_double,
-			    envelope_channel->decay->port_value.ags_port_double,
-			    envelope_channel->sustain->port_value.ags_port_double,
-			    envelope_channel->release->port_value.ags_port_double,
-			    envelope_channel->ratio->port_value.ags_port_double);
-}
-
 AgsRecall*
 ags_envelope_audio_signal_duplicate(AgsRecall *recall,
 				    AgsRecallID *recall_id,
@@ -266,6 +231,237 @@ ags_envelope_audio_signal_duplicate(AgsRecall *recall,
   return((AgsRecall *) envelope);
 }
 
+void
+ags_envelope_audio_signal_run_inter(AgsRecall *recall)
+{
+  AgsEnvelopeChannel *envelope_channel;
+  AgsEnvelopeAudioSignal *envelope_audio_signal;
+
+  AgsAudioSignal *source;
+  AgsNote *note;
+  
+  AgsComplex *attack;
+  AgsComplex *decay;
+  AgsComplex *sustain;
+  AgsComplex *release;  
+
+  GList *stream_source;
+
+  gdouble current_volume[4], current_ratio[4];
+  guint buffer_length[4];
+  gboolean buffer_on[4];
+
+  guint buffer_size;
+
+  guint i, j;
+
+  GValue audio_value = {0,};
+  GValue channel_value = {0,};
+
+  AGS_RECALL_CLASS(ags_envelope_audio_signal_parent_class)->run_inter(recall);
+
+  envelope_audio_signal = AGS_ENVELOPE_AUDIO_SIGNAL(recall);
+
+  source = AGS_RECALL_AUDIO_SIGNAL(envelope_audio_signal)->source;
+
+  stream_source = source->stream_current;
+
+  if(stream_source == NULL){
+    ags_recall_done(recall);
+    return;
+  }
+
+  /* initialize some control values */
+  for(i = 0; i < 4; i++){
+    current_volume[i] = 1.0;
+    current_ratio[i] = 1.0;
+    
+    buffer_length[i] = 0;
+    buffer_on[i] = FALSE;
+  }
+
+  buffer_size = source->buffer_size;
+
+  if(source->note != NULL){
+    gdouble x0, y0;
+    gdouble x1, y1;
+    gdouble current_x;
+    guint current_position;
+    guint current_frame, first_frame, buffer_size;
+    guint offset;
+    
+    note = source->note;
+
+    attack = &(note->attack);
+    decay = &(note->decay);
+    sustain = &(note->sustain);
+    release = &(note->release);
+
+    /* get offsets */
+    current_position = g_list_position(source->stream_beginning,
+				       source->stream_current);
+
+    if(current_position == 0){
+      current_frame = 0;
+      first_frame = source->first_frame;
+      buffer_size = source->buffer_size - source->first_frame;
+    }else{
+      current_frame = current_position * source->buffer_size - source->first_frame;
+      first_frame = 0;
+      buffer_size = source->buffer_size;
+    }
+
+    /* attack */
+    x0 = 0.0;
+    y0 = 0.0;
+
+    x1 = *attack[0];
+    y1 = *attack[1];
+
+    offset = (x1 - x0) * source->frame_count;
+    
+    if(offset < source->frame_count){
+      if(offset > current_frame){
+	buffer_on[0] = TRUE;
+
+	if(offset - current_frame > buffer_size){
+	  guint j;
+
+	  for(j = 0; j < floor((offset - current_frame) / source->buffer_size); j++);
+	  
+	  buffer_length[0] = offset - (j * source->buffer_size) - current_frame;
+
+	  current_x = source->frame_count / ((j * source->buffer_size) + current_frame);
+	}else{
+	  buffer_length[0] = offset - current_frame;
+
+	  current_x = source->frame_count / current_frame;
+	}
+
+	current_volume[0] = (y1 - y0) / (x1 - current_x);
+	current_ratio[0] = (y1 - y0) / (x1 - x0);
+      }
+    }else{
+      g_message("ags-envelope - invalid offset");
+    }
+    
+    /* decay */
+    x0 = *attack[0];
+    y0 = *attack[1];
+
+    x1 = *decay[0];
+    y1 = *decay[1];
+
+    offset += (x1 - x0) * source->frame_count;
+
+    if(offset < source->frame_count){
+      if(offset > current_frame){
+	buffer_on[1] = TRUE;
+
+	if(offset - current_frame > buffer_size){
+	  guint j;
+
+	  for(j = 0; j < floor((offset - current_frame) / source->buffer_size); j++);
+	  
+	  buffer_length[1] = offset - (j * source->buffer_size) - current_frame;
+
+	  current_x = source->frame_count / ((j * source->buffer_size) + current_frame);
+	}else{
+	  buffer_length[1] = offset - current_frame;
+
+	  current_x = source->frame_count / current_frame;
+	}
+
+	current_volume[1] = (y1 - y0) / (x1 - current_x);
+	current_ratio[1] = (y1 - y0) / (x1 - x0);
+      }
+    }else{
+      g_message("ags-envelope - invalid offset");
+    }
+
+    /* sustain */
+    x0 = *decay[0];
+    y0 = *decay[1];
+
+    x1 = *sustain[0];
+    y1 = *sustain[1];
+
+    offset += (x1 - x0) * source->frame_count;
+
+    if(offset < source->frame_count){
+      if(offset > current_frame){
+	buffer_on[2] = TRUE;
+
+	if(offset - current_frame > buffer_size){
+	  guint j;
+
+	  for(j = 0; j < floor((offset - current_frame) / source->buffer_size); j++);
+	  
+	  buffer_length[2] = offset - (j * source->buffer_size) - current_frame;
+
+	  current_x = source->frame_count / ((j * source->buffer_size) + current_frame);
+	}else{
+	  buffer_length[2] = offset - current_frame;
+
+	  current_x = source->frame_count / current_frame;
+	}
+
+	current_volume[2] = (y1 - y0) / (x1 - current_x);
+	current_ratio[2] = (y1 - y0) / (x1 - x0);
+      }
+    }else{
+      g_message("ags-envelope - invalid offset");
+    }
+    
+    /* release */
+    x0 = *sustain[0];
+    y0 = *sustain[1];
+
+    x1 = *release[0];
+    y1 = *release[1];
+    
+    if(offset < source->frame_count){
+      if(offset > current_frame){
+	buffer_on[3] = TRUE;
+
+	if(offset - current_frame > buffer_size){
+	  guint j;
+
+	  for(j = 0; j < floor((offset - current_frame) / source->buffer_size); j++);
+	  
+	  buffer_length[0] = offset - (j * source->buffer_size) - current_frame;
+
+	  current_x = source->frame_count / ((j * source->buffer_size) + current_frame);
+	}else{
+	  buffer_length[0] = offset - current_frame;
+
+	  current_x = source->frame_count / current_frame;
+	}
+
+	current_volume[0] = (y1 - y0) / (x1 - current_x);
+	current_ratio[0] = (y1 - y0) / (x1 - x0);
+      }
+    }else{
+      g_message("ags-envelope - invalid offset");
+    }
+  }else{
+    //TODO:JK: implement me
+    
+    return;
+  }
+  
+  for(i = 0, j = 0; i < 4; i++){
+    if(buffer_on[i]){
+      ags_audio_buffer_util_envelope(stream_source->data + j, 1,
+				     ags_audio_buffer_util_format_from_soundcard(source->format),
+				     current_volume[i],
+				     current_ratio[i]);
+
+      j += buffer_length[i];
+    }
+  }  
+}
+
 /**
  * ags_envelope_audio_signal_new:
  * @source: the source #AgsAudioSignal
