@@ -41,6 +41,7 @@ void ags_mute_channel_get_property(GObject *gobject,
 void ags_mute_channel_connect(AgsConnectable *connectable);
 void ags_mute_channel_disconnect(AgsConnectable *connectable);
 void ags_mute_channel_set_ports(AgsPlugin *plugin, GList *port);
+void ags_mute_channel_dispose(GObject *gobject);
 void ags_mute_channel_finalize(GObject *gobject);
 
 void ags_mute_channel_set_muted(AgsMutable *mutable, gboolean muted);
@@ -160,9 +161,17 @@ ags_mute_channel_class_init(AgsMuteChannelClass *mute_channel)
   gobject->set_property = ags_mute_channel_set_property;
   gobject->get_property = ags_mute_channel_get_property;
 
+  gobject->dispose = ags_mute_channel_dispose;
   gobject->finalize = ags_mute_channel_finalize;
 
   /* properties */
+  /**
+   * AgsMuteChannel:muted:
+   *
+   * The mute port.
+   * 
+   * Since: 0.7.122.7
+   */
   param_spec = g_param_spec_object("muted\0",
 				   "mute channel\0",
 				   "Mute the channel\0",
@@ -195,6 +204,7 @@ ags_mute_channel_init(AgsMuteChannel *mute_channel)
 				     "port-value-size\0", sizeof(gfloat),
 				     "port-value-length\0", 1,
 				     NULL);
+  g_object_ref(mute_channel->muted);
 
   mute_channel->muted->port_value.ags_port_float = (float) FALSE;
 
@@ -203,6 +213,7 @@ ags_mute_channel_init(AgsMuteChannel *mute_channel)
 
   /* add to port */
   port = g_list_prepend(port, mute_channel->muted);
+  g_object_ref(mute_channel->muted);
 
   /* set port */
   AGS_RECALL(mute_channel)->port = port;
@@ -311,6 +322,24 @@ ags_mute_channel_set_ports(AgsPlugin *plugin, GList *port)
   }
 }
 
+void
+ags_mute_channel_dispose(GObject *gobject)
+{
+  AgsMuteChannel *mute_channel;
+
+  mute_channel = AGS_MUTE_CHANNEL(gobject);
+
+  /* muted */
+  if(mute_channel->muted != NULL){
+    g_object_unref(G_OBJECT(mute_channel->muted));
+
+    mute_channel->muted = NULL;
+  }
+
+  /* call parent */
+  G_OBJECT_CLASS(ags_mute_channel_parent_class)->dispose(gobject);
+}
+
 void
 ags_mute_channel_finalize(GObject *gobject)
 {
@@ -318,6 +347,7 @@ ags_mute_channel_finalize(GObject *gobject)
 
   mute_channel = AGS_MUTE_CHANNEL(gobject);
 
+  /* muted */
   if(mute_channel->muted != NULL){
     g_object_unref(G_OBJECT(mute_channel->muted));
   }
