@@ -23,8 +23,6 @@
 #include <gdk/gdk.h>
 #include <pango/pangocairo.h>
 
-#include <libinstpatch/libinstpatch.h>
-
 #include "gsequencer_main.h"
 
 #include <ags/lib/ags_log.h>
@@ -69,6 +67,12 @@
 
 #include <ags/X/task/ags_simple_file_read.h>
 
+#include <ags/config.h>
+
+#ifdef AGS_WITH_LIBINSTPATCH
+#include <libinstpatch/libinstpatch.h>
+#endif
+
 #include <libxml/parser.h>
 #include <libxml/xlink.h>
 #include <libxml/xpath.h>
@@ -77,11 +81,14 @@
 #include <libxml/xmlmemory.h>
 #include <libxml/xmlsave.h>
 
+#include <pthread.h>
 #include <string.h>
 
 #include <unistd.h>
 #include <sys/types.h>
 
+#include <stdlib.h>
+
 #include "config.h"
 
 void ags_signal_handler(int signr);
@@ -200,9 +207,13 @@ ags_start_animation_thread(void *ptr)
 
   surface = cairo_image_surface_create_from_png(filename);
   image_data = cairo_image_surface_get_data(surface);
-  
-  bg_data = (unsigned char *) malloc(image_size * sizeof(unsigned char));
 
+  if(image_size > 0){
+    bg_data = (unsigned char *) malloc(image_size * sizeof(unsigned char));
+  }else{
+    bg_data = NULL;
+  }
+  
   if(image_data != NULL){
     memcpy(bg_data, image_data, image_size * sizeof(unsigned char));
   }
@@ -1373,7 +1384,9 @@ main(int argc, char **argv)
 				    NULL);
   }
   
+#ifdef AGS_WITH_LIBINSTPATCH
   ipatch_init();
+#endif
   //  g_log_set_fatal_mask("GLib-GObject\0", // "Gtk\0" G_LOG_DOMAIN, // 
 		       //		       G_LOG_LEVEL_CRITICAL); // G_LOG_LEVEL_WARNING
 
